# coding: utf-8

import os
import sys
import pathlib
import xmltodict
import package.MojXmlDef as MojXmlDef
import package.MojXmlPolygon as MojXmlPolygon
import package.MojXMLtoGeoJSON as MojXMLtoGeoJSON


def SaveGeoJson(src_file, exclude_flag, untreated_flag, treated_flag, daihyo_flag, convert_flag, chiban_flag):
    moj_obj = MojXml(src_file)

    mojGeojson = MojGeojson(moj_obj, exclude_flag, untreated_flag, treated_flag, daihyo_flag, src_file, convert_flag, chiban_flag)

    #地区外・別図のみのデータはdaihyo.geojsonを削除してgeojsonは出力しない
    dst_name2 = pathlib.Path(src_file).with_suffix(".daihyo.geojson").resolve()
    if os.path.exists(dst_name2) and os.path.getsize(dst_name2) < 140:
        os.remove(dst_name2)
        sys.exit()

    dst_name = pathlib.Path(src_file).with_suffix(".geojson").resolve()

    if chiban_flag is False:
        mojGeojson = mojGeojson.replace('null', '""') # 属性値に"null"があれば、""に置き換える
        with open(dst_name, 'w', encoding='utf-8') as f:
            f.write(mojGeojson)
            f.close()


# convert xml to object
def MojXml(src_file):
    with open(src_file, encoding='utf-8') as fp:
        xml_data = fp.read()
        moj_dict = xmltodict.parse(xml_data)

    version = moj_dict['地図']['version']
    city_name = moj_dict['地図']['市区町村名']
    crs = moj_dict['地図']['座標系']
    number_crs, named_crs = MojXmlDef.GetCrs(crs)
    datum_type = moj_dict.get('地図', {}).get('測地系判別')

    mojXmlPolygon = MojXmlPolygon.MojXmlPolygon(moj_dict)

    mojObj = {
        'version': version,
        'city_name': city_name,
        'crs': crs,
        'named_crs': named_crs,
        'number_crs': number_crs,
        'datum_type': datum_type,
        'mojXmlPolygon': mojXmlPolygon,
    }
    return mojObj


def MojGeojson(mojObj: dict, exclude_flag, untreated_flag, treated_flag, daihyo_flag, src_file, convert_flag, chiban_flag):
    return MojXMLtoGeoJSON.MojXMLtoGeoJSON(mojObj, exclude_flag, untreated_flag, treated_flag, daihyo_flag, src_file, convert_flag, chiban_flag)
